@@ -1,26 +1,21 @@
 package Net::Whois::Raw::Common;
-$Net::Whois::Raw::Common::VERSION = '2.100';
-# ABSTRACT: Helper for Net::Whois::Raw.
 
 use Encode;
-use warnings;
 use strict;
-use Regexp::IPv6 qw($IPv6_re);
-use Net::Whois::Raw::Data ();
-use Net::Whois::Raw ();
+require Net::Whois::Raw::Data;
 
 use utf8;
 
 # func prototype
 sub untaint(\$);
 
-# get whois from cache
+# get whois from cache 
 sub get_from_cache {
     my ($query, $cache_dir, $cache_time) = @_;
 
     return undef unless $cache_dir;
     mkdir $cache_dir unless -d $cache_dir;
-
+    
     my $now = time;
     # clear the cache
     foreach my $fn ( glob("$cache_dir/*") ) {
@@ -41,13 +36,13 @@ sub get_from_cache {
                 $result->[$level]->{text} = undef ;
             }
             else {
-                $result->[$level]->{text} = decode_utf8( $result->[$level]->{text} );
+        	$result->[$level]->{text} = decode_utf8( $result->[$level]->{text} );
             }
             $level++;
-            close $cache_fh;
+	    close $cache_fh;
         }
     }
-
+    
     return $result;
 }
 
@@ -57,96 +52,93 @@ sub write_to_cache {
 
     return unless $cache_dir && $result;
     mkdir $cache_dir unless -d $cache_dir;
-
+    
     untaint $query; untaint $cache_dir;
 
     my $level = 0;
     foreach my $res ( @{$result} ) {
-        local $res->{text} = $res->{whois} if not exists $res->{text};
+	local $res->{text} = $res->{whois} if not exists $res->{text};
 
-        next if defined $res->{text} && !$res->{text} || !defined $res->{text};
-        my $enc_text = $res->{text};
-        utf8::encode( $enc_text );
+	next if defined $res->{text} && !$res->{text} || !defined $res->{text};
+	utf8::encode( $res->{text} );
         my $postfix = sprintf("%02d", $level);
         if ( open( my $cache_fh, '>', "$cache_dir/$query.$postfix" ) ) {
             print $cache_fh $res->{srv} ? $res->{srv} :
                 ( $res->{server} ? $res->{server} : '')
                 , "\n";
-
-            print $cache_fh $enc_text ? $enc_text : '';
-
+                
+            print $cache_fh $res->{text} ? $res->{text} : '';
+            
             close $cache_fh;
             chmod 0666, "$cache_dir/$query.$postfix";
         }
         $level++;
     }
-
+    
 }
 
+
 # remove copyright messages, check for existance
 sub process_whois {
-    my ( $query, $server, $whois, $CHECK_FAIL, $OMIT_MSG, $CHECK_EXCEED ) = @_;
+    my ($query, $server, $whois, $CHECK_FAIL, $OMIT_MSG, $CHECK_EXCEED) = @_;
 
     $server = lc $server;
-    my ( $name, $tld ) = split_domain( $query );
+    my ($name, $tld) = split_domain($query);
 
     # use string as is
     no utf8;
 
     if ( $CHECK_EXCEED ) {
-        my $exceed = $Net::Whois::Raw::Data::exceed{ $server };
+        my $exceed = $Net::Whois::Raw::Data::exceed{$server};
 
         if ( $exceed && $whois =~ /$exceed/s) {
-            return $whois, 'Connection rate exceeded';
+            return $whois, "Connection rate exceeded";
         }
     }
-
-    $whois = _strip_trailer_lines( $whois )  if $OMIT_MSG;
-
+    
     if ( $CHECK_FAIL || $OMIT_MSG ) {
+    
+	my %notfound = %Net::Whois::Raw::Data::notfound;
+	my %strip = %Net::Whois::Raw::Data::strip;
+    
+	my $notfound = $notfound{$server};
+    
+	my @strip = $strip{$server} ? @{$strip{$server}} : ();
+	my @lines;
+MAIN:
+	foreach (split(/\n/, $whois)) {
+	    if ( $CHECK_FAIL && $notfound && /$notfound/ ) {
+        	return undef, "Not found";
+	    };
+	    if ($OMIT_MSG) {
+        	foreach my $re (@strip) {
+		    next MAIN if (/$re/);
+        	}
+	    }
+	    s/^\s+//;
+        
+	    push(@lines, $_);
+	}
+    
+	$whois = join "\n", @lines, '';
 
-        my %notfound = %Net::Whois::Raw::Data::notfound;
-        my %strip = %Net::Whois::Raw::Data::strip;
-
-        my $notfound = $notfound{ $server };
-
-        my @strip = $strip{ $server } ? @{ $strip{ $server } } : ();
-        my @lines;
-
-        MAIN:
-        for ( split /\n/, $whois ) {
-            if ( $CHECK_FAIL && $notfound && /$notfound/ ) {
-                return undef, "Not found";
-            }
-
-            if ( $OMIT_MSG ) {
-                for my $re ( @strip ) {
-                    next MAIN  if /$re/;
-                }
-            }
-
-            push @lines, $_;
-        }
-
-        $whois = join "\n", @lines, '';
-
-        if ( $OMIT_MSG ) {
-            $whois =~ s/(?:\s*\n)+$/\n/s;
-            $whois =~ s/^\n+//s;
-            $whois =~ s|\n{3,}|\n\n|sg;
-        }
+	if ( $OMIT_MSG ) {
+	    $whois =~ s/(?:\s*\n)+$/\n/s;
+	    $whois =~ s/^\n+//s;
+	    $whois =~ s|\n{3,}|\n\n|sg;
+	}
+    }
+    
+    if ( defined $Net::Whois::Raw::Data::postprocess{$server} ) {
+        $whois = $Net::Whois::Raw::Data::postprocess{$server}->($whois);
     }
 
-    if ( defined $Net::Whois::Raw::Data::postprocess{ $server } ) {
-        $whois = $Net::Whois::Raw::Data::postprocess{ $server }->( $whois );
+    if ( defined $Net::Whois::Raw::POSTPROCESS{$server} ) {
+        $whois = $Net::Whois::Raw::POSTPROCESS{$server}->($whois);
     }
 
-    if ( defined $Net::Whois::Raw::POSTPROCESS{ $server } ) {
-        $whois = $Net::Whois::Raw::POSTPROCESS{ $server }->( $whois );
-    }
-
-    if ( defined $Net::Whois::Raw::Data::codepages{ $server } ) {
-        $whois = decode( $Net::Whois::Raw::Data::codepages{ $server }, $whois );
+    if ( defined $Net::Whois::Raw::Data::codepages{$server} ) {
+        $whois = decode( $Net::Whois::Raw::Data::codepages{$server}, $whois );
     }
     else {
         utf8::decode( $whois );
@@ -155,54 +147,47 @@ sub process_whois {
     return $whois, undef;
 }
 
-# Tries to strip trailer lines of whois
-sub _strip_trailer_lines {
-    my ( $whois ) = @_;
 
-    for my $re ( @Net::Whois::Raw::Data::strip_regexps ) {
-        $whois =~ s/$re/$1/;
-    }
-
-    return $whois;
-}
-
-# get whois-server for domain / tld
+# get whois-server for domain
 sub get_server {
-    my ($dom, $is_ns, $tld) = @_;
-
-    $tld ||= get_dom_tld( $dom );
-    $tld = uc $tld;
+    my ($dom, $USE_CNAME) = @_;
+    
+    my $tld = uc get_dom_tld( $dom );
 
     if ( grep { $_ eq $tld } @Net::Whois::Raw::Data::www_whois ) {
         return 'www_whois';
     }
 
-    my $srv = '';
-    if ( $is_ns ) {
-        $srv = $Net::Whois::Raw::Data::servers{ $tld . '.NS' } ||
-               $Net::Whois::Raw::Data::servers{ 'NS' };
-    }
-    else {
-            #my $cname = "$tld.whois-servers.net";
-            my $cname = "whois.nic.$tld";
-        $srv = $Net::Whois::Raw::Data::servers{ $tld } || $cname;
-    }
+    my $cname = "$tld.whois-servers.net";
+    my $srv = $Net::Whois::Raw::Data::servers{$tld} || $cname;
+    $srv = $cname if $USE_CNAME && gethostbyname($cname);
 
     return $srv;
 }
 
 sub get_real_whois_query{
-    my ( $whoisquery, $srv, $is_ns ) = @_;
+    my ($whoisquery, $srv, $is_ns) = @_;
+    
+    $is_ns = 1 if $whoisquery =~ s/.NS$//i;
 
-    $srv .= '.ns'  if $is_ns;
-
-    if ( $srv eq 'whois.crsnic.net' && domain_level( $whoisquery ) == 2 ) {
-        return "domain $whoisquery";
+    if ($srv eq 'whois.crsnic.net' && domain_level($whoisquery) == 2) {
+        $whoisquery = "domain $whoisquery";
     }
-    elsif ( $Net::Whois::Raw::Data::query_prefix{ $srv } ) {
-        return $Net::Whois::Raw::Data::query_prefix{ $srv } . $whoisquery;
+    elsif ($srv eq 'whois.denic.de') {
+        $whoisquery = "-T dn,ace -C ISO-8859-1 $whoisquery";
     }
-
+    elsif ($srv eq 'whois.nic.name') {
+        if ( $is_ns ) {
+            $whoisquery = "nameserver=$whoisquery";
+	}
+	else {
+	    $whoisquery = "domain=$whoisquery";
+	}
+    }
+    elsif ( $is_ns && $srv eq 'whois.nsiregistry.net' ) {
+        $whoisquery = "nameserver = $whoisquery";
+    }
+    
     return $whoisquery;
 }
 
@@ -211,13 +196,13 @@ sub get_dom_tld {
     my ($dom) = @_;
 
     my $tld;
-    if ( is_ipaddr($dom) || is_ip6addr($dom) ) {
+    if ( is_ipaddr($dom) ) {
         $tld = "IP";
     }
     elsif ( domain_level($dom) == 1 ) {
         $tld = "NOTLD";
     }
-    else {
+    else { 
         my @alltlds = keys %Net::Whois::Raw::Data::servers;
         @alltlds = sort { dlen($b) <=> dlen($a) } @alltlds;
         foreach my $awailtld (@alltlds) {
@@ -228,7 +213,7 @@ sub get_dom_tld {
         }
         unless ($tld) {
             my @tokens = split(/\./, $dom);
-            $tld = $tokens[-1];
+            $tld = $tokens[-1]; 
         }
     }
 
@@ -239,111 +224,147 @@ sub get_dom_tld {
 # %param: domain*
 sub get_http_query_url {
     my ($domain) = @_;
-
+    
     my ($name, $tld) = split_domain($domain);
     my @http_query_data;
     # my ($url, %form);
 
-    my $server = get_server( undef, undef, $tld );
-
-    if ($tld eq 'ru' || $tld eq 'su') {
+    if ($tld eq 'tv') {
         my $data = {
-            url  => "http://www.nic.ru/whois/?domain=$name.$tld",
+            url  => "http://www.tv/cgi-bin/whois.cgi?domain=$name&tld=tv",
             form => '',
         };
         push @http_query_data, $data;
     }
-    elsif ($tld eq 'ip') {
+    elsif ($tld eq 'mu') {
         my $data = {
-            url  => "http://www.nic.ru/whois/?ip=$name",
-            form => '',
-        };
-        push @http_query_data, $data;
-    }
-    elsif ($tld eq 'ws') {
-        my $data = {
-            url  => "http://worldsite.ws/utilities/lookup.dhtml?domain=$name&tld=$tld",
-            form => '',
-        };
-        push @http_query_data, $data;
-    }
-    elsif ($tld eq 'kz') {
-        my $data = {
-            url  => "http://www.nic.kz/cgi-bin/whois?query=$name.$tld&x=0&y=0",
-            form => '',
-        };
-        push @http_query_data, $data;
-    }
-    elsif ($tld eq 'vn') {
-        # VN doesn't have web whois at the moment...
-        my $data = {
-            url  => "http://www.tenmien.vn/jsp/jsp/tracuudomain1.jsp",
+            url  => 'http://www.mu/cgi-bin/mu_whois.cgi',
             form => {
-                cap2        => ".$tld",
-                referer     => 'http://www.vnnic.vn/english/',
-                domainname1 => $name,
+                whois => $name,
             },
         };
         push @http_query_data, $data;
     }
-    elsif ($tld eq 'ac') {
+    elsif ($tld eq 'spb.ru' || $tld eq 'msk.ru') {
         my $data = {
-            url  => "http://nic.ac/cgi-bin/whois?query=$name.$tld",
-            form => '',
+	    url  => "http://www.relcom.ru/Services/Whois/?fullName=$name.$tld",
+	    form => '',
+        };
+        push @http_query_data, $data;
+    }
+    elsif ($tld eq 'ru' || $tld eq 'su') {
+	my $data = {
+	    url  => "http://www.nic.ru/whois/?domain=$name.$tld",
+	    form => '',
+	};
+	push @http_query_data, $data;
+    }
+    elsif ($tld eq 'ip') {
+	my $data = {
+	    url  => "http://www.nic.ru/whois/?ip=$name",
+	    form => '',
+	};
+	push @http_query_data, $data;
+    }
+    elsif ($tld eq 'in') {
+	my $data = {
+	    url  => "http://www.registry.in/cgi-bin/whois.cgi?whois_query_field=$name",
+	    form => '',
+        };
+        push @http_query_data, $data;
+    }
+    elsif ($tld eq 'cn') {
+        my $data = {
+	    url  => "http://ewhois.cnnic.net.cn/whois?value=$name.$tld&entity=domain",
+	    form => '',
+        };
+        push @http_query_data, $data;
+    }
+    elsif ($tld eq 'ws') {
+	my $data = {
+	    url  => "http://worldsite.ws/utilities/lookup.dhtml?domain=$name&tld=$tld",
+	    form => '',
+        };
+        push @http_query_data, $data;
+    }
+    elsif ($tld eq 'kz') {
+	my $data = {
+	    url  => "http://www.nic.kz/cgi-bin/whois?query=$name.$tld&x=0&y=0",
+	    form => '',
+        };
+        push @http_query_data, $data;
+    }
+    elsif ($tld eq 'vn') {
+	my $data = {
+	    url  => "http://www.vnnic.vn/jsp/jsp/tracuudomain1.jsp",
+	    form => {
+		cap2        => ".$tld",
+		referer     => 'http://www.vnnic.vn/english/',
+		domainname1 => $name,
+	    },
+    	};
+    	push @http_query_data, $data;
+    }
+    elsif ($tld eq 'ac') {
+	my $data = {
+	    url  => "http://nic.ac/cgi-bin/whois?query=$name.$tld",
+	    form => '',
         };
         push @http_query_data, $data;
     }
     elsif ($tld eq 'bz') {
-        my $data = {
-            url  => "http://www.test.bz/Whois/index.php?query=$name&output=nice&dotname=.$tld&whois=Search",
+	my $domcode = unpack( 'H*', "$name.$tld" );
+	my $data = {
+	    url  => 'http://www.belizenic.bz/cgi-bin/Registrar_YTest?action=whois&action2=whois&domain='.$domcode,
+	    form => '',
         };
         push @http_query_data, $data;
     }
     elsif ($tld eq 'tj') {
-        #my $data = {
-        #    url  => "http://get.tj/whois/?lang=en&domain=$domain",
-        #    from => '',
-        #};
-        #push @http_query_data, $data;
+	my $data = {
+	    url  => "http://get.tj/whois/?lang=en&domain=$domain",
+	    from => '',
+	};
+	push @http_query_data, $data;
 
-        # first level on nic.tj
-        #$data = {
-        #    url  => "http://www.nic.tj/cgi/lookup2?domain=$name",
-        #    from => '',
-        #};
-        #push @http_query_data, $data;
+	# first level on nic.tj
+	$data = {
+	    url  => "http://www.nic.tj/cgi/lookup2?domain=$name",
+	    from => '',
+	};
+	push @http_query_data, $data;
 
-        # second level on nic.tj
-        my $data = {
-            url  => "http://www.nic.tj/cgi/whois?domain=$name",
-            from => '',
-        };
-        push @http_query_data, $data;
+	# second level on nic.tj
+	$data = {
+	    url  => "http://www.nic.tj/cgi/whois?domain=$name",
+	    from => '',
+	};
+	push @http_query_data, $data;
+	
+	#$data = {
+	#    url  => "http://ns1.nic.tj/cgi/whois?domain=$name",
+	#    from => '',
+	#};
+	#push @http_query_data, $data;
 
-        #$data = {
-        #    url  => "http://ns1.nic.tj/cgi/whois?domain=$name",
-        #    from => '',
-        #};
-        #push @http_query_data, $data;
-
-        #$data = {
-        #    url  => "http://62.122.137.16/cgi/whois?domain=$name",
-        #    from => '',
-        #};
-        #push @http_query_data, $data;
+	#$data = {
+	#    url  => "http://62.122.137.16/cgi/whois?domain=$name",
+	#    from => '',
+	#};
+	#push @http_query_data, $data;
     }
-
+        
     # return $url, %form;
     return \@http_query_data;
 }
 
 sub have_reserve_url {
     my ( $tld ) = @_;
-
+    
     my %tld_list = (
         'tj' => 1,
     );
-
+    
     return defined $tld_list{$tld};
 }
 
@@ -351,15 +372,69 @@ sub have_reserve_url {
 # %param: resp*, tld*
 sub parse_www_content {
     my ($resp, $tld, $url, $CHECK_EXCEED) = @_;
-
-    my $server = get_server( undef, undef, $tld );
-
+     
     chomp $resp;
     $resp =~ s/\r//g;
 
     my $ishtml;
 
-    if ( $tld eq 'ru' || $tld eq 'su' ) {
+    if ($tld eq 'tv') {
+
+        $resp = decode_utf8( $resp );
+        
+        return 0 unless
+            $resp =~ /(<TABLE BORDER="0" CELLPADDING="4" CELLSPACING="0" WIDTH="95%">.+?<\/TABLE>)/is;
+        $resp = $1;
+        $resp =~ s/<BR><BR>.+?The data in The.+?any time.+?<BR><BR>//is;
+        return 0 if $resp =~ /Whois information is not available for domain/s;
+        $ishtml = 1;
+
+    }
+    elsif ( $tld eq 'spb.ru' || $tld eq 'msk.ru' ) {
+    
+        $resp = decode( 'koi8-r', $resp  );
+
+        return undef unless $resp =~ m|<TABLE BORDER="0" CELLSPACING="0" CELLPADDING="2"><TR><TD BGCOLOR="#990000"><TABLE BORDER="0" CELLSPACING="0" CELLPADDING="20"><TR><TD BGCOLOR="white">(.+?)</TD></TR></TABLE></TD></TR></TABLE>|s;
+        $resp = $1;
+
+        return 0 if $resp =~ m/СВОБОДНО/;
+
+        if ($resp =~ m|<PRE>(.+?)</PRE>|s) {
+            $resp = $1;
+        }
+        elsif ($resp =~ m|DNS \(name-серверах\):</H3><BLOCKQUOTE>(.+?)</BLOCKQUOTE><H3>Дополнительную информацию можно получить по адресу:</H3><BLOCKQUOTE>(.+?)</BLOCKQUOTE>|) {
+            my $nameservers = $1;
+            my $emails = $2;
+            my (@nameservers, @emails);
+            while ($nameservers =~ m|<CODE CLASS="h2black">(.+?)</CODE>|g) {
+                push @nameservers, $1;
+            }
+            while ($emails =~ m|<CODE CLASS="h2black"><A HREF=".+?">(.+?)</A></CODE>|g) {
+                push @emails, $1;
+            }
+            if (scalar @nameservers && scalar @emails) {
+                $resp = '';
+                foreach my $ns (@nameservers) {
+                    $resp .= "nserver:      $ns\n";
+                }
+                foreach my $email (@emails) {
+                    $resp .= "e-mail:       $email\n";
+                }
+            }
+        }
+        
+    }
+    elsif ($tld eq 'mu') {
+
+        $resp = decode_utf8( $resp );
+
+        return 0 unless
+            $resp =~ /(<p><b>Domain Name:<\/b><br>.+?)<hr width="75%">/s;
+        $resp = $1;
+        $ishtml = 1;
+
+    }
+    elsif ( $tld eq 'ru' || $tld eq 'su' ) {
 
         $resp = decode( 'koi8-r', $resp );
 
@@ -372,48 +447,81 @@ sub parse_www_content {
 
         $resp = 'ERROR' if $resp =~ m/Error:/i || $resp !~ m/Информация о домене .+? \(по данным WHOIS.RIPN.NET\):/;
         #TODO: errors
-
+        
     }
     elsif ($tld eq 'ip') {
 
         $resp = decode_utf8( $resp );
-
+        
         return 0 unless $resp =~ m|<p ID="whois">(.+?)</p>|s;
 
         $resp = $1;
-
+        
         $resp =~ s|<a.+?>||g;
         $resp =~ s|</a>||g;
         $resp =~ s|<br>||g;
         $resp =~ s|&nbsp;| |g;
 
     }
-    elsif ($tld eq 'ws') {
+    elsif ($tld eq 'in') {
 
         $resp = decode_utf8( $resp );
 
-        if ($resp =~ /Whois information for .+?:(.+?)<table>/s) {
+        if ( $resp =~ /Domain ID:\w{3,10}-\w{4}\n(.+?)\n\n/s ) {
             $resp = $1;
-            $resp =~ s|<font.+?>||isg;
-            $resp =~ s|</font>||isg;
-
-            $ishtml = 1;
-        }
-        else {
+            $resp =~ s/<br>//g;
+        } 
+	else {
             return 0;
         }
 
     }
-    elsif ($tld eq 'kz') {
+    elsif ($tld eq 'cn') {
 
         $resp = decode_utf8( $resp );
 
-        if ($resp =~ /Domain Name\.{10}/s && $resp =~ /<pre>(.+?)<\/pre>/s) {
+        if ($resp =~ m|<table border=1 cellspacing=0 cellpadding=2>\n\n(.+?)\n</table>|s) {
             $resp = $1;
+            $resp =~ s|<a.+?>||isg;
+            $resp =~ s|</a>||isg;
+            $resp =~ s|<font.+?>||isg;
+            $resp =~ s|</font>||isg;
+            $resp =~ s|<tr><td class="t_blue">.+?</td><td class="t_blue">||isg;
+            $resp =~ s|</td></tr>||isg;
+            $resp =~ s|\n\s+|\n|sg;
+            $resp =~ s|\n\n|\n|sg;
         }
-        else {
+	else {
             return 0;
         }
+
+    }
+    elsif ($tld eq 'ws') {
+
+        $resp = decode_utf8( $resp );
+
+	if ($resp =~ /Whois information for .+?:(.+?)<table>/s) {
+	    $resp = $1;
+            $resp =~ s|<font.+?>||isg;
+            $resp =~ s|</font>||isg;
+
+            $ishtml = 1;
+	}
+	else {
+	    return 0;
+	}
+
+    }
+    elsif ($tld eq 'kz') {
+    
+        $resp = decode_utf8( $resp );
+
+	if ($resp =~ /Domain Name\.{10}/s && $resp =~ /<pre>(.+?)<\/pre>/s) {
+	    $resp = $1;
+	}
+	else {
+	    return 0;
+	}
     }
     elsif ($tld eq 'vn') {
 
@@ -422,19 +530,19 @@ sub parse_www_content {
         if ($resp =~ /\(\s*?(Domain .+?:\s*registered)\s*?\)/i )  {
             $resp = $1;
         }
-        else {
+	else {
             return 0;
         }
 
         #
-        # if ($resp =~/#ENGLISH.*?<\/tr>(.+?)<\/table>/si) {
-        #    $resp = $1;
-        #    $resp =~ s|</?font.*?>||ig;
-        #    $resp =~ s|&nbsp;||ig;
-        #    $resp =~ s|<br>|\n|ig;
-        #    $resp =~ s|<tr>\s*<td.*?>\s*(.*?)\s*</td>\s*<td.*?>\s*(.*?)\s*</td>\s*</tr>|$1 $2\n|isg;
-        #    $resp =~ s|^\s*||mg;
-        #
+	# if ($resp =~/#ENGLISH.*?<\/tr>(.+?)<\/table>/si) {
+	#    $resp = $1;
+	#    $resp =~ s|</?font.*?>||ig;
+	#    $resp =~ s|&nbsp;||ig;
+	#    $resp =~ s|<br>|\n|ig;
+	#    $resp =~ s|<tr>\s*<td.*?>\s*(.*?)\s*</td>\s*<td.*?>\s*(.*?)\s*</td>\s*</tr>|$1 $2\n|isg;
+	#    $resp =~ s|^\s*||mg;
+	# 
     }
     elsif ($tld eq 'ac') {
 
@@ -443,7 +551,7 @@ sub parse_www_content {
         if ($CHECK_EXCEED && $resp =~ /too many requests/is) {
             die "Connection rate exceeded";
         }
-        elsif ($resp =~ /<!--- Start \/ Domain Info --->(.+?)<!--- End \/ Domain Info --->/is) {
+	elsif ($resp =~ /<!--- Start \/ Domain Info --->(.+?)<!--- End \/ Domain Info --->/is) {
             $resp = $1;
             $resp =~ s|</?table.*?>||ig;
             $resp =~ s|</?b>||ig;
@@ -453,7 +561,7 @@ sub parse_www_content {
             $resp =~ s|</?td>||ig;
             $resp =~ s|^\s*||mg;
         }
-        else {
+	else {
             return 0;
         }
 
@@ -462,27 +570,15 @@ sub parse_www_content {
 
         $resp = decode_utf8( $resp );
 
-        if ( $resp =~ m{
-                <blockquote>
-                (.+)
-                </blockquote>
-            }xms )
-        {
-            $resp = $1;
-            if ( $resp =~ /NOT\s+FOUND/ || $resp =~ /No\s+Domain/ ) {
-                # Whois info not found
-                return 0;
-            }
+	if ($resp =~ m|<pre>(.+?)</pre>|xms) {
+	    $resp = $1;
+	}
 
-            $resp =~ s|<[^<>]+>||ig;
-        }
-        else {
-            return 0;
-        }
     }
     elsif ( $tld eq 'tj' && $url =~ m|^http\://get\.tj| ) {
+    
         $resp = decode_utf8( $resp );
-
+        
         if ($resp =~ m|<!-- Content //-->\n(.+?)<!-- End Content //-->|s ) {
             $resp = $1;
             $resp =~ s|<[^<>]+>||ig;
@@ -506,26 +602,27 @@ sub parse_www_content {
     elsif ( $tld eq 'tj' && $url =~ m|\.nic\.tj/cgi/lookup| ) {
 
         $resp = decode_utf8( $resp );
-
+        
         if ($resp =~ m|<div[0-9a-z=\" ]*>\n?(.+?)\n?</div>|s) {
             $resp = $1;
 
-            return 0 if $resp =~ m|may be available|s;
+	    return 0 if $resp =~ m|may be available|s;
 
             $resp =~ s|\n\s+|\n|sg;
             $resp =~ s|\s+\n|\n|sg;
             $resp =~ s|\n\n|\n|sg;
             $resp =~ s|<br.+||si;
         }
-        else {
+	else {
             return 0;
         }
-
+        
     }
     elsif ( $tld eq 'tj' && $url =~ m|\.nic\.tj/cgi/whois| || $url =~ m|62\.122\.137\.16| ) {
-        $resp = decode_utf8( $resp );
 
-        if ( $resp =~ m{ <table [^>]*? > (.+) (:? </table> ) }sxmi ) {
+        $resp = decode_utf8( $resp );
+        
+        if ($resp =~ m|<table[0-9a-z=\" ]*>\n(.+?)\n</table>|s) {
             $resp = $1;
             $resp =~ s|</?tr>||ig;
             $resp =~ s|<td>| |ig;
@@ -538,15 +635,15 @@ sub parse_www_content {
             $resp =~ s|\s+\n|\n|sg;
             $resp =~ s|\n\n|\n|sg;
         }
-        else {
+	else {
             return 0;
         }
-
+        
     }
     else {
         return 0;
     }
-
+    
     return $resp;
 }
 
@@ -557,19 +654,16 @@ sub is_ipaddr {
 
 # check, if it's IPv6-address?
 sub is_ip6addr {
-    my ( $ip ) = @_;
-
-    return 0 unless defined $ip;
-
-    return $ip =~ /^$IPv6_re$/;
+    # TODO: bad implementation!!!!!
+    $_[0] =~ /:/;
 }
 
 # get domain level
 sub domain_level {
     my ($str) = @_;
-
+    
     my $dotcount = $str =~ tr/././;
-
+    
     return $dotcount + 1;
 }
 
@@ -581,11 +675,11 @@ sub split_domain {
 
     my $name;
     if (uc $tld eq 'IP' || $tld eq 'NOTLD') {
-        $name = $dom;
+	$name = $dom;
     }
     else {
-        $dom =~ /(.+?)\.$tld$/; # or die "Can't match $tld in $dom";
-        $name = $1;
+	$dom =~ /(.+?)\.$tld$/; # or die "Can't match $tld in $dom";
+	$name = $1;
     }
 
     return ($name, $tld);
@@ -607,30 +701,3 @@ sub untaint (\$) {
 }
 
 1;
-
-__END__
-
-=pod
-
-=encoding UTF-8
-
-=head1 NAME
-
-Net::Whois::Raw::Common - Helper for Net::Whois::Raw.
-
-=head1 VERSION
-
-version 2.100
-
-=head1 AUTHOR
-
-Alexander Nalobin <alexander@nalobin.ru>
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is copyright (c) 2002-2016 by Alexander Nalobin.
-
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
-
-=cut
